/*
 * Copyright (c) 2011 Lucas Baudin <xapantu@gmail.com>
 *
 * Euclide is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * Euclide is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; see the file COPYING.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */

namespace Euclide
{
    /**
     * A function to check if the path is a valid bazaar branch. It doesn't
     * work for remote branch, like lp:euclide
     * 
     * @param path a path, e.g. "." or "/home/user/repository"
     * 
     * @returns whether the path is a bzr branch
     **/
    public bool is_bzr_branch(string path) {
        try {
            int status = 0;
            string output;
            Process.spawn_sync (path, {"bzr", "info"}, null, SpawnFlags.SEARCH_PATH, null, out output, out output, out status);
            return status == 0;
        } catch (SpawnError e) {
            warning (e.message);
            return false;
        }
    }
}
