/*
 * Copyright (c) 2011 Lucas Baudin <xapantu@gmail.com>
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; see the file COPYING.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */


public class Euclide.CMake.Cache : Euclide.CMake.BaseParser
{
    public const string BUILD_TYPE = "CMAKE_BUILD_TYPE:STRING";
    
    Gee.HashMap<string,string> command_map;
    public Cache(string filename_)
    {
        filename = filename_;
        load_content();
        parse_file();
        generate_map();
    }
    
    public string? get_value(string key)
    {
        if(command_map.has_key(key))
        {
            return command_map[key];
        }
        return null;
    }
    
    public void set_value(string key, string value)
    {
        foreach(var command in commands)
        {
            if(command is Assignment)
            {
                Assignment assignment = (Assignment)command;
                if(assignment.key == key) assignment.assignment_value = value;
            }
        }
        save_content();
        generate_map();
    }

    public void save_content()
    {
        string content = "";
        foreach(var command in commands)
        {
            content += command.command;
        }
        try
        {
            FileUtils.set_contents(filename, content);
        }
        catch(FileError e)
        {
            critical("Couldn't write the CMakeLists.txt: %s", e.message);
        }
    }

    void generate_map()
    {
        command_map = new Gee.HashMap<string,string>();
        foreach(var command in commands)
        {
            if(command is Assignment)
            {
                Assignment assignment = (Assignment)command;
                command_map[assignment.key] = assignment.assignment_value;
            }
        }
    }
}
