/*
 * Copyright (c) 2011 Lucas Baudin <xapantu@gmail.com>
 *
 * Euclide is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * Euclide is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; see the file COPYING.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */

/**
 * A simple class to store files informations, it is just used by the Binary
 * to store their source file. It provides some utilities to have the relative
 * and absolute path.
 **/
public class Euclide.CMake.File : GLib.Object
{
    public string relative { get; protected set; }
    public string absolute { get; protected set; }
    public string relative_dir { get; protected set; }
    public string name { get; protected set; }

    /**
     * Create a new File object.
     * 
     * @param relative the content of the relative properties, tabs will
     * be removed from it name.
     **/
    public File(string relative, string absolute)
    {
        this.relative = relative.replace("\t", "");
        this.absolute = absolute.replace("\t", "");
        relative_dir = "";
        
        /* FIXME: this code could be cleaner using some GLib.File */
        name = relative.split("/")[relative.split("/").length - 1];
        foreach(var dir in relative.split("/"))
        {
            if(dir != name)
                relative_dir += dir + "/";
        }
    }
}

public class Euclide.CMake.Binary : GLib.Object
{
    public string name;
    public string filename;
    /**
     * Don't add/remove files manually, use add_file and remove_file ;)
     **/
    public Gee.ArrayList<File> files;
    public string source_dir;
    public Gee.ArrayList<Command> related_functions = new Gee.ArrayList<Command>();
    public ValaPrecompile? vala_precompile
    {
        set { related_functions.add(value); }
        get
        {
            foreach(var function in related_functions)
            {
                if(function is ValaPrecompile)
                {
                    return function as ValaPrecompile;
                }
            }
            return null;
        }
    }
    
    public void add_file(string name)
    {
        files.add(new Euclide.CMake.File(name, source_dir + "/" + name));
        vala_precompile.files.append(name);
        vala_precompile.construct_command();
    }
    
    public void remove_file(string name)
    {
        vala_precompile.remove_file(name);
        vala_precompile.construct_command();
        
        foreach(var file in files)
        {
            if(file.relative == name)
            {
                files.remove(file);
                return;
            }
        }
    }

    public Binary(string filename, string name)
    {
        this.filename = filename;
        this.name = name;
        files = new Gee.ArrayList<File>();
    }
    
    /*
     * FIXME: This function needs a complete refactoring, it is buggy and a bit useless ATM
     */
    public void update (string? new_package = null, string? old_package = null) {
        //ValaPrecompile vala_precompile = this.vala_precompile;
        return;
    }
    
    public void add_dep(string dep, bool is_internal = false)
    {
        if(!is_internal)
        {
            vala_precompile.add_package(dep);
            bool pkg_config_found = false;
            foreach(var target_function in get_deps())
            {
                var pkg_functions = new List<PkgCheckModules>();
                target_function.get_pkg_config_ancestors(ref pkg_functions);
                foreach(var pkg_function in pkg_functions)
                {
                    pkg_config_found = true;
                    pkg_function.packages.append(dep);
                    pkg_function.construct_command();
                    pkg_function.parse();
                }
            }
            if(!pkg_config_found)
            {
                critical("Add pkg_config things is not implemented yet");
            }
        }
        else
        {
            critical("Internal deps not implemented yet!");
        }
    }
    
    public void remove_dep(string dep, bool is_internal = false)
    {
        if(!is_internal)
        {
            /* for vala_precompile */
            vala_precompile.remove_package(dep);
            
            /* Now, we remove the dep in all pkg_check_modules related to this binary
             * FIXME: what happens if two binaries uses the same pkg_check_modules call?
             */
            foreach(var target_function in get_deps())
            {
                var pkg_functions = new List<PkgCheckModules>();
                target_function.get_pkg_config_ancestors(ref pkg_functions);
                foreach(var pkg_function in pkg_functions)
                {
                    pkg_function.remove_package(dep);
                    pkg_function.construct_command();
                    pkg_function.parse();
                }
            }
        }
        else
        {
            critical("Internal deps not implemented yet!");
        }
    }

    public Gee.ArrayList<string> get_internal_dependencies()
    {
        Gee.ArrayList<string> deps_list = new Gee.ArrayList<string>();
        foreach(var command in related_functions)
        {
            if(command is TargetLinkLibraries)
            {
                /* We need to let the first arg, since in 
                 * target_link_libraries, it is the target name
                 */
                int j = 0;
                foreach(var dep in ((TargetLinkLibraries)command).args)
                {
                    if(j > 0 && !dep.contains("$")) /* we suppose that all variables are
                                                     * externals deps (which is wrong, FIXME).
                                                     */
                        deps_list.add(dep);
                    j++;
                }
            }
        }
        return deps_list;
    }
    
    public Gee.ArrayList<string> get_dependencies()
    {
        /* First, we get all target_link_libraries */
        Gee.ArrayList<TargetFunction> deps = get_deps();
        Gee.ArrayList<string> deps_list = new Gee.ArrayList<string>();
        foreach(var target_function in deps)
        {
            /* We need to analysis all functions which define variable for
             * them, as pkg_check_modules */
            var pkg_functions = new List<PkgCheckModules>();
            target_function.get_pkg_config_ancestors(ref pkg_functions);
            foreach(var pkg_function in pkg_functions)
            {
                foreach(var dep in pkg_function.packages)
                {
                    deps_list.add(dep);
                }
            }
        }
        return deps_list;
    }

    public Gee.ArrayList<TargetFunction> get_deps()
    {
        var to_return = new Gee.ArrayList<TargetFunction>();
        foreach(var command in related_functions)
        {
            if(command is TargetLinkLibraries)
            {
                TargetFunction function = command as TargetLinkLibraries;
                to_return.add(function);
            }
        }
        return to_return;
    }
}
