/*
 * Copyright (c) 2011 Lucas Baudin <xapantu@gmail.com>
 *
 * Euclide is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * Euclide is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; see the file COPYING.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */

public class Euclide.CMake.Project : GLib.Object
{
    public string name;
    public string cmake_version;
    public string cmake_policy;
    public bool is_vala = true;
    public Gee.ArrayList<string> pkg;
    public Gee.ArrayList<string> files;
    public Gee.ArrayList<Binary> bin;
    public Gee.ArrayList<Binary> lib;
    public string project_path;
    public string build_type;

    public Project()
    {
        pkg = new Gee.ArrayList<string>();
        files = new Gee.ArrayList<string>();
        bin = new Gee.ArrayList<Binary>();
        lib = new Gee.ArrayList<Binary>();
        cmake_version = "VERSION 2.6";
        cmake_policy = "VERSION 2.6";
    }

    /**
     * A utility function which read the executables/libraries list, and which
     * returns the binary named "name".
     * 
     * @param name the binary name
     *
     * @returns A binary named name, or null if it is not found.
     **/
    public Binary? get_target(string name)
    {
        foreach(var binary in bin)
        {
            if(binary.name == name)
            {
                return binary;
            }
        }

        foreach(var binary in lib)
        {
            if(binary.name == name)
            {
                return binary;
            }
        }
        
        return null; /* not found */
    }

    public static void create(string project_path)
    {
        /* first, create our project path */
        GLib.File file = GLib.File.new_for_path(project_path);
        try
        {
            Process.spawn_sync(file.get_parent().get_path(), {"quickly", "create", "vala-template", file.get_basename()}, null, SpawnFlags.SEARCH_PATH, null);
        }
        catch(Error e)
        {
            print("Is Quickly installed?%s\n", e.message);
        }
    }
}
