/*
 * Copyright (c) 2011 Lucas Baudin <xapantu@gmail.com>
 *
 * Euclide is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * Euclide is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; see the file COPYING.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */

using Gtk;
using Granite.Widgets;

public class Euclide.NewProjectDialog : Dialog
{
    FileChooserButton location_button;
    Entry project_name;
    Label preview_location;
    
    RadioButton check_executable;
    
    RadioButton check_library;
    
    public NewProjectDialog(Window parent)
    {
        set_parent(parent);
        set_title("New Project");
        add_buttons(Stock.CANCEL, ResponseType.CANCEL, "Create", ResponseType.OK, null);

        add_select_location();
        add_choose_name();
        add_options();
    }

    public string? create()
    {
        show_all();
        int reponse = run();
        if(reponse == ResponseType.OK)
        {
            CMake.Project.create(location_button.get_filename() + "/" + project_name.get_text());
            return (location_button.get_filename() + "/" + project_name.get_text());
        }
        return null;
    }
    
    void add_select_location()
    {
        VBox vbox = (VBox)get_content_area();
        var hbox_select_location = new HBox(false, 5);
        hbox_select_location.pack_start(new Label("Location:"), false, false);
        
        location_button = new FileChooserButton("Project", FileChooserAction.SELECT_FOLDER);
        hbox_select_location.pack_start(location_button);
        vbox.pack_start(hbox_select_location);
    }
    
    void add_choose_name()
    {
        VBox vbox = (VBox)get_content_area();
        var hbox_choose_name = new HBox(false, 5);
        hbox_choose_name.pack_start(new Label("Name:"), false, false);
        
        project_name = new HintedEntry("Project Name…");
        var label = new Gtk.Label("");
        hbox_choose_name.pack_start(project_name);
        
        preview_location = new Label(location_button.get_filename() + "/" + project_name.get_text());
        project_name.changed.connect(() => {
            project_name.text = project_name.text.replace(" ", "-").replace("_", "-");
            preview_location.label = location_button.get_filename() + "/" + project_name.get_text();
            set_response_sensitive(ResponseType.OK, true);
            label.label = "";
            if(FileUtils.test(preview_location.label, FileTest.EXISTS))
            {
                label.label = "This directory already exists.\nPlease choose a different project name/directory.";
                set_response_sensitive(ResponseType.OK, false);
            }
        });
        location_button.current_folder_changed.connect(() => {
            preview_location.label = location_button.get_filename() + "/" + project_name.get_text();
            set_response_sensitive(ResponseType.OK, true);
            label.label = "";
            if(FileUtils.test(preview_location.label, FileTest.EXISTS))
            {
                label.label = "This directory already exists.\nPlease choose a different project name/directory.";
                set_response_sensitive(ResponseType.OK, false);
            }
        });
        preview_location.set_sensitive(false);

        vbox.pack_start(hbox_choose_name);
        vbox.pack_start(preview_location);
        vbox.pack_start(label);
    }
    
    void add_options()
    {
        VBox vbox = (VBox)get_content_area();
        
        /* Executable */
        var hbox = new HBox(false, 5);
        check_executable = new RadioButton.with_label(null, "Executable");
        hbox.pack_start(check_executable);
        vbox.pack_start(hbox);
        
        /* Library */
        hbox = new HBox(false, 5);
        check_library = new RadioButton.with_label(check_executable.get_group(), "Library");
        check_library.sensitive = false;
        hbox.pack_start(check_library);
        vbox.pack_start(hbox);
        
    }
}
