/*
 * Copyright (c) 2011 Lucas Baudin <xapantu@gmail.com>
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; see the file COPYING.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */

namespace Euclide.Output
{
    public enum ResultType
    {
        TEXT,
        PROGRESS,
        WARNING,
        ERROR
    }
    public struct ParserResult
    {
        ResultType type;
        int value;
        int line;
        string file;
        string text;
    }
    public Gee.ArrayList<ParserResult?> parse(string text)
    {
        var results = new Gee.ArrayList<ParserResult?>();
        ParserResult? result = null;
        /* We'll only parse the first line */
        foreach(var line in text.split("\n"))
        {
            switch(line[0])
            {
            case '[': /* progress notification */
                result = ParserResult();
                result.type = ResultType.PROGRESS;
                result.value = int.parse(line.split("]")[0].replace("%", "").replace("[", ""));
                results.add(result);
                break;
            case '/': /* valac warning */
                result = ParserResult();
                result.type = ResultType.WARNING;
                result.file = line.split(":")[0];
                int length = line.split(":").length;
                if(length == 4)
                {
                    result.line = int.parse(line.split(":")[1].split(".")[0]);
                    if(line.split(":")[2] == " error")
                    {
                        result.type = ResultType.ERROR;
                    }
                    result.text = line.split(":")[3];
                    results.add(result);
                }
                else if(length == 5)
                {
                    result.line = int.parse(line.split(":")[1]);
                    if(line.split(":")[3] == " error")
                    {
                        result.type = ResultType.ERROR;
                    }
                    result.text = line.split(":")[4];
                    results.add(result);
                }
                break;
            }
        }
        
        return results;
    }
}
