/*
 * Copyright (c) 2011 Lucas Baudin <xapantu@gmail.com>
 *
 * This is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; see the file COPYING.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */

using Gtk;

public class Euclide.Terminal : TreeView
{
    public signal void open_file(string file, int line);
    
    public ListStore store;
    
    public Terminal()
    {
			store = new ListStore (5, typeof(Gdk.Pixbuf), typeof(int), typeof(string), typeof(string), typeof(string));
			this.set_model (store);
			
			this.insert_column_with_attributes (-1, "Type", new CellRendererPixbuf (), "pixbuf", 0);
			this.insert_column_with_attributes (-1, "Message", new CellRendererText (), "text", 2);
			this.insert_column_with_attributes (-1, "Line", new CellRendererText (), "text", 1);
			this.insert_column_with_attributes (-1, "File", new CellRendererText (), "text", 3);
			
			//jump to file/line on single click
			this.cursor_changed.connect ( (path) => {
				TreeIter iter;
				this.get_selection ().get_selected (null, out iter);
				Value line;
				Value file;
				this.store.get_value (iter, 1, out line);
				this.store.get_value (iter, 4, out file);
	            open_file (file.get_string (), line.get_int ());
		    });
		    
		    //remove the message on doubelclick
		    this.row_activated.connect ( () => {
		    	TreeIter iter;
		    	this.get_selection ().get_selected (null, out iter);
		    	store.remove (iter);
		    });
    }
    
    
    public void clear()
    {
        store.clear ();
    }
    
    public void add_error(string text, string file, int line, string project_path)
    {
        var icon = render_icon(Gtk.Stock.DIALOG_ERROR, Gtk.IconSize.MENU, null);
        add_item(text, file, line, icon, project_path);
    }
    public void add_warning(string text, string file, int line, string project_path)
    {
        var icon = render_icon(Gtk.Stock.DIALOG_WARNING, Gtk.IconSize.MENU, null);
        add_item(text, file, line, icon, project_path);
    }
    public void add_question(string text, string file, int line, string project_path)
    {
        var icon = render_icon(Gtk.Stock.DIALOG_QUESTION, Gtk.IconSize.MENU, null);
        add_item(text, file, line, icon, project_path);
    }
    
    void add_item(string text, string file, int line, Gdk.Pixbuf icon, string project_path)
    {
        Gtk.TreeIter iter;
		store.append(out iter);
		store.set(iter, 0, icon, 1, line, 2, text, 
		     3, file.replace(project_path + "/", ""), 4, file);
    }
}

public class Euclide.RawTerminal : Vte.Terminal
{
    public RawTerminal()
    {
        set_scroll_on_output(true);
        /*editable = false;
        modify_font(Pango.FontDescription.from_string("Monospace 11"));
        wrap_mode = Gtk.WrapMode.WORD;*/
    }
    
    public void add_text(string text)
    {
        feed(text + "\r", -1);
    }
    public void clear()
    {
        reset(true, true);
    }
}
