/*
 * Copyright (c) 2011 Lucas Baudin <xapantu@gmail.com>
 *
 * Euclide is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * Euclide is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; see the file COPYING.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */

public class Euclide.CMake.File : GLib.Object
{
    public string relative { get; protected set; }
    public string absolute { get; protected set; }
    public string relative_dir { get; protected set; }
    public string name { get; protected set; }
    public File(string relative, string absolute)
    {
        this.relative = relative;
        this.absolute = absolute;
        relative_dir = "";
        
        name = relative.split("/")[relative.split("/").length - 1];
        foreach(var dir in relative.split("/"))
        {
            if(dir != name)
                relative_dir += dir + "/";
        }
    }
}

public class Euclide.CMake.Binary : GLib.Object
{
    public string name;
    public string filename;
    public Gee.ArrayList<File> files;
    public string source_dir;
    public Command vala_precompile;
    
    public Binary (string filename, string name) {
        this.filename = filename;
        this.name = name;
        files = new Gee.ArrayList<File>();
    }
    
    public void update () {
        Gee.ArrayList<string> options = new Gee.ArrayList<string>();
        Gee.ArrayList<string> packages = new Gee.ArrayList<string>();
        
        
        int state = -1; /* var_name, PACKAGES: 1, OPTIONS: 2, FILES: 3; */
        foreach(var arg in vala_precompile.split_args)
        {
            if(arg == "PACKAGES")
            {
                state = 1;
            }
            else if(arg == "OPTIONS")
            {
                state = 2;
            }
            else if(state == -1)
            {
                state = 3;
            }
            else if(state == 1)
            {
                packages.add(arg);
            }
            else if(state == 2)
            {
                options.add(arg);
            }
        }
        
        vala_precompile.command = "vala_precompile(" + vala_precompile.split_args[0] + "\n";
        foreach(var file in files)
        {
            vala_precompile.command += file.relative + "\n";
        }
        vala_precompile.command += "PACKAGES\n";
        foreach(var file in packages)
        {
            vala_precompile.command += file + "\n";
        }
        vala_precompile.command += "OPTIONS\n";
        foreach(var file in options)
        {
            vala_precompile.command += file + "\n";
        }
        vala_precompile.command += ")";
    }
}
