/*
 * Copyright (c) 2011 Lucas Baudin <xapantu@gmail.com>
 *
 * This is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; see the file COPYING.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */

using Euclide;
using Plugins;
using Scratch;

Gdk.Pixbuf? pix_class = null;
Gdk.Pixbuf? pix_namespace = null;
Gdk.Pixbuf? pix_function = null;
Gdk.Pixbuf? pix_struct = null;
Gdk.Pixbuf? pix_enum = null;

public class Euclide.Plugins.Symbols : Peas.Activatable, Object {
    Gtk.TreeStore store;
    Gtk.TreeView view;
    
    MainWindow? window = null;
    Widgets.SplitView? split_view = null;
    Gtk.Notebook? notebook_context = null;
    
    string old_text = "";
    
    Scratch.Services.Interface plugins;
    
    public Object object { owned get; construct; }

    public Symbols() {
       
    }
    
    public void activate () {
        plugins = (Scratch.Services.Interface) object;
        plugins.hook_window.connect (on_new_window);
        plugins.hook_notebook_context.connect (on_notebook_context);        
    }

    public void deactivate()
    {
        view.destroy ();
    }
    
    public void update_state()
    {
    }
    
    void on_new_window (MainWindow window) {
        this.window = window;
        this.split_view = this.window.split_view;
                
    }


    public void on_notebook_context (Gtk.Notebook notebook) {
        
        if (this.notebook_context != null)
            return;
        this.notebook_context = notebook;
        
        pix_class = new Gdk.Pixbuf.from_file(Euclide.DATADIR + "/plugins/euclidesymbols/class.png");
        pix_namespace = new Gdk.Pixbuf.from_file(Euclide.DATADIR + "/plugins/euclidesymbols/namespace.png");
        pix_function = new Gdk.Pixbuf.from_file(Euclide.DATADIR + "/plugins/euclidesymbols/method.png");
        pix_struct = new Gdk.Pixbuf.from_file(Euclide.DATADIR + "/plugins/euclidesymbols/struct.png");
        pix_enum = new Gdk.Pixbuf.from_file(Euclide.DATADIR + "/plugins/euclidesymbols/enum.png");

        create_store();
        set_up_widget();
        Gtk.ScrolledWindow scrolled = new Gtk.ScrolledWindow(null, null);
        scrolled.width_request = 100;
        scrolled.add(view);
        scrolled.show_all();
        
        this.notebook_context.append_page (scrolled, new Gtk.Label ("Symbols"));
        
        Timeout.add (1000, () => {
            var doc = this.window.get_current_document ();
            if (doc == null) return true;
            Gtk.TextBuffer buffer = doc.source_view.buffer;
            if(buffer != null) {
                lock (store) {
                    if (old_text != buffer.text) {
                        store.clear ();
                        old_text = buffer.text;
                        FileParser.parse_string (store, old_text);
                        view.expand_all ();
                        view.show_all ();
                    }
                }
            }
            return true;
        });
    }

    void create_store () {
        debug("Creating store");
        store = new Gtk.TreeStore(4, typeof(string), typeof(SymbolType), typeof(int), typeof(Gdk.Pixbuf));
        store.set_sort_column_id(0, Gtk.SortType.ASCENDING);
    }
        
    void set_up_widget()
    {
        view = new Gtk.TreeView.with_model(store);
        var pix_render = new Gtk.CellRendererPixbuf();
        var renderer = new Gtk.CellRendererText();
        renderer.ellipsize = Pango.EllipsizeMode.END;
        var column = new Gtk.TreeViewColumn();
        column.pack_start(pix_render, false);
        column.pack_start(renderer, true);
        column.set_attributes(pix_render, "pixbuf", 3, null);
        column.set_attributes(renderer, "text", 0, null);
        view.append_column(column);
        view.headers_visible = false;
        view.button_release_event.connect(on_tree_view_click);
        view.rules_hint = true;
    }
    
    bool on_tree_view_click(Gdk.EventButton button)
    {
        lock(store)
        {
            Gtk.TreeSelection selection = view.get_selection();
            Gtk.TreeIter? iter = Gtk.TreeIter();
            if(selection.get_selected(null, out iter))
            {
                int line = 0;
                store.get(iter, 2, out line);
                this.window.get_current_document ().source_view.go_to_line (line);
            }
        }
        return false;
    }
}

public int main(string[] args)
{
    Test.init(ref args);
    
    Test.add_func("/euclide/plugins/symbols/parse_block/check_child", () => {
        string test_basic = """
        public class test {
            void function_1()
            {
                test_truc();
                test_blah () ;
            }

            function_2()
            {
                {
                }
            }

            function_3(args1, args2)
            {
                {
                }
            }
        }
        """;

        Gtk.TreeStore symbols = new Gtk.TreeStore(4, typeof(string), typeof(SymbolType), typeof(int), typeof(Gdk.Pixbuf));
        int word = 0;
        SymbolType s_type = SymbolType.NONE;
        int s_line = 0;
        string name = "";
        Gtk.TreeIter parent_iter = Gtk.TreeIter();
        Gtk.TreeIter iter = Gtk.TreeIter();

        int line_number = FileParser.parse_block(symbols, null, 0, ref word, test_basic.split("\n"), 0);
        //assert(line_number == 13);
        assert(symbols.get_iter_first(out parent_iter) == true);
        symbols.get(parent_iter, 0, out name, 1, out s_type, 2, out s_line);
        assert(name == "test");
        assert(s_type == SymbolType.CLASS);
        assert(s_line == 1);

        assert(symbols.iter_nth_child(out iter, parent_iter, 0) == true);
        symbols.get(iter, 0, out name, 1, out s_type, 2, out s_line);
        assert(name == "function_1");
        assert(s_type == SymbolType.FUNCTION);
        assert(s_line == 3); /* FIXME: should be 2 but we count at the { */
        
        assert(symbols.iter_nth_child(out iter, parent_iter, 1) == true);
        symbols.get(iter, 0, out name, 1, out s_type, 2, out s_line);
        assert(name == "function_2");
        assert(s_type == SymbolType.FUNCTION);
        assert(s_line == 9);
        
        assert(symbols.iter_nth_child(out iter, parent_iter, 2) == true);
        symbols.get(iter, 0, out name, 1, out s_type, 2, out s_line);
        assert(name == "function_3");
        assert(s_type == SymbolType.FUNCTION);
        assert(s_line == 15);
        assert(symbols.iter_next(ref iter) == false);

        word = 0;
        symbols.clear();
        line_number = FileParser.parse_block(symbols, null, 2, ref word, test_basic.split("\n"), 1);
        assert(line_number == 6);
    });
    
    Test.add_func("/euclide/plugins/symbols/parse_block/complex_1", () => {
        string test_basic = """
        public class test {
            void function_1()
            {
                test_truc();
                test_blah ();
                
                list.open_file.connect( (s) => {open_file(s);});
            }

            function_2()
            {
                {
                }
            }

            function_3(args1, args2)
            {
                {
                }
            }
        }
        """;

        Gtk.TreeStore symbols = new Gtk.TreeStore(4, typeof(string), typeof(SymbolType), typeof(int), typeof(Gdk.Pixbuf));
        int word = 0;
        SymbolType s_type = SymbolType.NONE;
        int s_line = 0;
        string name = "";
        Gtk.TreeIter parent_iter = Gtk.TreeIter();
        Gtk.TreeIter iter = Gtk.TreeIter();

        int line_number = FileParser.parse_block(symbols, null, 0, ref word, test_basic.split("\n"), 0);
        //assert(line_number == 13);
        assert(symbols.get_iter_first(out parent_iter) == true);
        symbols.get(parent_iter, 0, out name, 1, out s_type, 2, out s_line);
        assert(name == "test");
        assert(s_type == SymbolType.CLASS);
        assert(s_line == 1);

        assert(symbols.iter_nth_child(out iter, parent_iter, 0) == true);
        symbols.get(iter, 0, out name, 1, out s_type, 2, out s_line);
        assert(name == "function_1");
        assert(s_type == SymbolType.FUNCTION);
        assert(s_line == 3); /* FIXME: should be 2 but we count at the { */

        assert(symbols.iter_nth_child(out iter, parent_iter, 1) == true);
        symbols.get(iter, 0, out name, 1, out s_type, 2, out s_line);
        assert(name == "function_2");
        assert(s_type == SymbolType.FUNCTION);

    });
    
    Test.add_func("/euclide/plugins/symbols/parse_block/comments", () => {
        string test_basic = """
        /* This could go in an external lib */
        
        /**
         * This class is a wrapper to run an async command. It provides useful signals.
         **/
        public class test {

        }
        """;

        Gtk.TreeStore symbols = new Gtk.TreeStore(4, typeof(string), typeof(SymbolType), typeof(int), typeof(Gdk.Pixbuf));
        int word = 0;
        string name = "";
        Gtk.TreeIter parent_iter = Gtk.TreeIter();

        int line_number = FileParser.parse_block(symbols, null, 0, ref word, test_basic.split("\n"), 0);
        assert(symbols.get_iter_first(out parent_iter) == true);
        symbols.get(parent_iter, 0, out name);
        assert(name == "test");

    });
    
    Test.add_func("/euclide/plugins/symbols/parse_block/namespace", () => {
        string test_basic = """
        namespace Euclide.test
        {
            string test = "} }";
            public class test
            {
            string test = "}";
                void test_func(args)
                {
                }

            }
        }
        """;

        Gtk.TreeStore symbols = new Gtk.TreeStore(4, typeof(string), typeof(SymbolType), typeof(int), typeof(Gdk.Pixbuf));
        int word = 0;
        string name = "";
        SymbolType s_type = SymbolType.NONE;
        Gtk.TreeIter parent_iter = Gtk.TreeIter();
        Gtk.TreeIter iter = Gtk.TreeIter();
        Gtk.TreeIter iter_child = Gtk.TreeIter();

        int line_number = FileParser.parse_block(symbols, null, 0, ref word, test_basic.split("\n"), 0);
        assert(symbols.get_iter_first(out parent_iter) == true);
        symbols.get(parent_iter, 0, out name);
        assert(name == "Euclide.test");

        assert(symbols.iter_nth_child(out iter, parent_iter, 0) == true);
        symbols.get(iter, 0, out name, 1, out s_type);
        assert(name == "test");
        assert(s_type == SymbolType.CLASS);

        assert(symbols.iter_nth_child(out iter_child, iter, 0) == true);
        symbols.get(iter_child, 0, out name, 1, out s_type);
        assert(name == "test_func");
        assert(s_type == SymbolType.FUNCTION);

    });
    
    Test.add_func("/euclide/plugins/symbols/parse_block/only_functions", () => {
        string test_basic = """
        void test_func(args)
        {
        }
        """;

        Gtk.TreeStore symbols = new Gtk.TreeStore(4, typeof(string), typeof(SymbolType), typeof(int), typeof(Gdk.Pixbuf));
        int word = 0;
        string name = "";
        SymbolType s_type = SymbolType.NONE;
        Gtk.TreeIter parent_iter = Gtk.TreeIter();
        Gtk.TreeIter iter = Gtk.TreeIter();

        FileParser.parse_string(symbols, test_basic);
        assert(symbols.get_iter_first(out parent_iter) == true);
        symbols.get(parent_iter, 0, out name);
        assert(name == "test_func");

    });
    
    Test.add_func("/euclide/plugins/symbols/parse_block/structs", () => {
        string test_basic = """
        public struct test
        {
            test 1;
            test 2;
        }
        """;

        Gtk.TreeStore symbols = new Gtk.TreeStore(4, typeof(string), typeof(SymbolType), typeof(int), typeof(Gdk.Pixbuf));
        int word = 0;
        string name = "";
        SymbolType s_type = SymbolType.NONE;
        Gtk.TreeIter parent_iter = Gtk.TreeIter();
        Gtk.TreeIter iter = Gtk.TreeIter();

        FileParser.parse_string(symbols, test_basic);
        assert(symbols.get_iter_first(out parent_iter) == true);
        symbols.get(parent_iter, 0, out name, 1, out s_type);
        assert(name == "test");
        assert(s_type == SymbolType.STRUCT);

    });
    
    Test.add_func("/euclide/plugins/symbols/other_file", () => {
        string content = null;
        FileUtils.get_contents("../symbols.vala", out content);
        int word = 0;
        Gtk.TreeStore symbols = new Gtk.TreeStore(4, typeof(string), typeof(SymbolType), typeof(int), typeof(Gdk.Pixbuf));
        FileParser.parse_string(symbols, content);
    });
    
    Test.add_func("/euclide/plugins/symbols/other_file_class", () => {
        string content = null;
        FileUtils.get_contents("../test-Threads.vala", out content);
        int word = 0;
        string name = "";
        Gtk.TreeIter parent_iter = Gtk.TreeIter();
        Gtk.TreeStore symbols = new Gtk.TreeStore(4, typeof(string), typeof(SymbolType), typeof(int), typeof(Gdk.Pixbuf));
        FileParser.parse_string(symbols, content);

        assert(symbols.get_iter_first(out parent_iter) == true);
        symbols.get(parent_iter, 0, out name);
        assert(name == "Euclide.Threads.SimpleCommand");
    });
    Test.run();
    return 0;
}

[ModuleInit]
public void peas_register_types (GLib.TypeModule module) {
    var objmodule = module as Peas.ObjectModule;
    objmodule.register_extension_type (typeof (Peas.Activatable),
                                       typeof (Euclide.Plugins.Symbols));
}
