
public class ValaAnjutaInterface
{
	public Gtk.SourceView editor { get; set; }
	public File file { get; set; }
	public File project_dir { get; internal set; }
	
	public signal void saved (File file);
	public signal void char_added (Gtk.TextIter position, string ch);
	
	public ValaPlugin plugin;
	Tip tip_window;
	
	public ValaAnjutaInterface ()
	{
		plugin = new ValaPlugin (this);
		tip_window = new Tip (this);
	}
	
	public bool set_current_file (File _file)
	{
		file = _file;
		var ret = plugin.set_current_document (file);
		return ret;
	}
	
	public Gtk.TextIter get_line_begin_position (int line)
	{
		Gtk.TextIter iter;
		editor.buffer.get_iter_at_line (out iter, line);
		while (!iter.starts_word ())
			iter.forward_char ();
		
		return iter;
	}
	
	public Gtk.TextIter get_position ()
	{
		Gtk.TextIter current_position;
		editor.buffer.get_iter_at_offset (out current_position, editor.buffer.cursor_position);
		return current_position;
	}
	
	public int get_lineno ()
	{
		return get_position ().get_line ();
	}
	
	public int get_column ()
	{
		return get_position ().get_line_offset ();
	}
	
	public string get_text (Gtk.TextIter start, Gtk.TextIter end)
	{
		return start.get_text (end);
	}
	
	public void show_tips (List<string> tips, Gtk.TextIter pos)
	{
		if (tips.length () == 0)
			return;
		
		string list = "";
		foreach (var tip in tips)
			list += tip;
		
		tip_window.open (list, pos);
	}
	
	public void hide_tip ()
	{
		tip_window.hide ();
	}
}

public class Tip : Gtk.Window
{
	Gtk.Label _tip;
	public string tip {
		get {
			return _tip.label;
		}
		set {
			_tip.label = value;
		}
	}
	
	ValaAnjutaInterface vala_interface;
	
	public Tip (ValaAnjutaInterface _vala_interface)
	{
		Object (type : Gtk.WindowType.POPUP, type_hint : Gdk.WindowTypeHint.TOOLTIP);
		
		_tip = new Gtk.Label ("");
		vala_interface = _vala_interface;
		
		add (_tip);
	}
	
	public void open (string tips, Gtk.TextIter iter)
	{
		tip = tips;
		resize (1, 1);
		
		int x, y, winx, winy, ox, oy, view_width, label_width, label_height;
		Gdk.Rectangle rect;
		
		var window = vala_interface.editor.get_window (Gtk.TextWindowType.TEXT);
		
		vala_interface.editor.get_iter_location (iter, out rect);
		vala_interface.editor.buffer_to_window_coords (Gtk.TextWindowType.TEXT, rect.x, rect.y, out winx, out winy);
		window.get_origin (out ox, out oy);
		
		x = winx + ox;
		y = winy + oy;
		
		window.get_geometry (null, null, out view_width, null);
		_tip.get_size_request (out label_width, out label_height);
		var width_left = (ox + view_width ) - (x + label_width);
		if (width_left < 0)
			x += width_left;
		
		y -= label_height + 25;
		
		move (x, y);
		show_all ();
	}
}

public class ValaProposal : Object, Gtk.SourceCompletionProposal
{
	
	string name;
	public void *data;
	
	public ValaProposal (string _name, void *_data)
	{
		data = _data;
		name = _name;
	}
	
	public string get_label ()
	{
		return name;
	}
	public string get_info ()
	{
		return name;
	}
	public string get_markup ()
	{
		return name;
	}
	public string get_text ()
	{
		return name;
	}
}
