/*
 * Copyright (c) 2011 Lucas Baudin <xapantu@gmail.com>
 *
 * This is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; see the file COPYING.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */


using Scratch;
using Scratch.Services;

/* TODO: ouch, this code doesn't look good... */
public Document? current_document = null;
public Gtk.SourceView? current_view = null;
public Gtk.SourceBuffer? current_buffer = null;

public class Euclide.Plugins.Completion : Peas.ExtensionBase,  Peas.Activatable {
    
    MainWindow main_window;

    public Object object { owned get; construct; }
    Scratch.Services.Interface plugins;

    uint timeout_parse = -1;
    
    public void activate()
    {
        plugins = (Scratch.Services.Interface) object;
        timeout_parse = Timeout.add(5000, on_timeout_update);

        plugins.hook_window.connect ((w) => {
            this.main_window = w;
        });
        plugins.hook_document.connect (on_new_source_view);
        
    }

    public void deactivate()
    {
        if(timeout_parse > 0) Source.remove(timeout_parse);
    }

    public void update_state()
    {
    }
    

    Gtk.TreeStore store;

    Gtk.Window window;
    Gtk.TreeView tree_view;
    static const unichar[] stoppers = {' ', '\n', '(', ';', '}', '{', '.'};

    Gtk.CellRendererText cell;

    void create_window () {
        window = Object.new(typeof(Gtk.Window),
                            transient_for: this.main_window,
                            type: Gtk.WindowType.POPUP) as Gtk.Window;
        window.resizable = true;
        window.can_focus = false;
        window.accept_focus = false;
        window.skip_pager_hint = true;
        window.skip_taskbar_hint = true;
        window.decorated = false;
        window.set_keep_above(true);
        var scrolled = new Gtk.ScrolledWindow(null, null);
        scrolled.hscrollbar_policy = Gtk.PolicyType.NEVER;
        window.add(scrolled);
        window.type_hint = Gdk.WindowTypeHint.NORMAL;
        window.move(100, 200);

        store = new Gtk.TreeStore(2, typeof(string), typeof(string), null);
        tree_view = new Gtk.TreeView.with_model(store);
        scrolled.add(tree_view);
        cell = new Gtk.CellRendererText();
        cell.ellipsize = Pango.EllipsizeMode.END;
        var settings = new GLib.Settings("org.pantheon.scratch.settings");
        cell.font = settings.get_string("font");
        var column = new Gtk.TreeViewColumn.with_attributes("Function",
                                                            cell, "text", 0, null);
        tree_view.append_column(column);
        tree_view.headers_visible = false;
        window.width_request = 200;
        tree_view.row_activated.connect(on_selection_changed_tree);
        tree_view.show_expanders = false;
    }
    
    List<Gtk.TextView> text_view = new List<Gtk.TextView>();

    public void on_new_source_view(Scratch.Services.Document doc)
    {
        if (doc == null)
            return;
        
        // Globals vars
        current_document = doc;
        current_view = doc.source_view;
        current_buffer = doc.source_view.buffer;
        
        var view = doc.source_view;
        //assert(view != null);
        view.key_press_event.connect(on_key_press);
        //view.focus_out_event.connect(() => { window.hide(); return false; });
        //view.button_press_event.connect( () => { window.hide(); return false;});
        text_view.append(view);
         
        // Provider
        current_view.completion.get_providers ().foreach ((p) => { 
            try {
                current_view.completion.remove_provider (p); 
            } catch (Error e) {
                warning (e.message);
            }
        });
        
        var vala_interface = new ValaAnjutaInterface ();
        vala_interface.editor = view;
        vala_interface.file = doc.file;
		vala_interface.project_dir = detect_project (doc.file);
        
        var comp_provider = new ValaProvider (vala_interface.plugin);

        try {
            current_view.completion.add_provider (comp_provider);
        } catch (Error e) { 
            warning (e.message); 
        }
        
        //let's have a quick look if we can find a CMakeLists.txt with some packages
		var cmake = detect_project (doc.file);

		if (cmake.query_exists ()) {
			string content = "";
			size_t len = 0;
			
			try {
				FileUtils.get_contents (cmake.get_path (), out content, out len);
			} catch (Error e) { 
			    warning (e.message); 
			    return;
			}
			
			var place = content.index_of ("PACKAGES");
			
			if (place != -1) {
				int end = place + 8;
				string character = "";
				do {
					end = end + 1;
					character = content.get (end).to_string ();
				//package names are small caps, the vala_precompile delimiter not
				} while (character.down () == character && character != ")");
				
				var rel = content.substring (place + 8, end - place - 8);
				
				var regex = /\s+/;
				var splits = regex.split (rel);
				
				foreach (var split in splits) {
					if (split == "")
						continue;
					vala_interface.plugin.add_package (split);
				}
			}
		}
    }
    
    const string [] vcss = {".bzr", ".git", ".hg"};
    
    // Detect a CMakeLists.txt file to get pkg deps
    File? detect_project (File opened)
	{
		var cmake = File.new_for_path (Path.build_filename (opened.get_parent ().get_path (), "CMakeLists.txt"));
		if (cmake.query_exists ())
		    return cmake;
		
		//go up looking for a vcs indicating folder
		var dir = opened;
		while ((dir = dir.get_parent ()) != null) {
			foreach (var vcs in vcss) {
				if (dir.get_child (vcs).query_exists ()) {
					var file = File.new_for_path (Path.build_filename (dir.get_path (), "CMakeLists.txt"));
					debug (file.get_path ());
					return file;
				}
			}
		}
		
		//checking for src, might not be under version control yet
		dir = opened.get_parent ();
		if (dir.get_basename () == "src") {
			dir = File.new_for_path (Path.build_filename (dir.get_parent ().get_path (), "CMakeLists.txt"));
		} else if (dir.get_parent ().get_basename () == "src") {
			dir = File.new_for_path (Path.build_filename (dir.get_parent ().get_parent ().get_path (), "CMakeLists.txt"));
		}
		return dir;
	}
    
    bool on_timeout_update () {
        try {
            unowned Thread<void*> thread_a = Thread.create<void*> (threaded_update, true);
            thread_a.set_priority(ThreadPriority.LOW);
            return true;
        } catch (ThreadError e) {
            warning (e.message);
            return false;
        }
    }
    
    void* threaded_update () {
        /*parser.clear();
        foreach(var view in text_view)
        {
            if(view != null)
                parser.parse_text_view(view);
        }*/
        return null;
    }
    
    void on_selection_changed_tree(Gtk.TreeView view, Gtk.TreePath path, Gtk.TreeViewColumn col)
    {
        on_selection_changed(view, path, col);
    }
    bool on_selection_changed(Gtk.TreeView view, Gtk.TreePath path, Gtk.TreeViewColumn col)
    {
        Gtk.TreeSelection selection = view.get_selection();
        Gtk.TreeIter iter;
        var model = (Gtk.TreeModel)store;
        if(selection.get_selected(out model, out iter))
        {
            string response;
            store.get(iter, 1, out response, -1);
            response = response.slice(last_to_find.length, response.length);
            this.main_window.get_current_document ().source_view.get_buffer().insert_at_cursor(response, -1);
            window.hide();
            return true;
        }
        return false;
    }
    
    void update_window_size(int record)
    {
        int height;
        cell.get_preferred_height(window, null, out height);
        window.resize(200, int.min(height * record + 10, 600));
    }
    
    bool on_key_press(Gtk.Widget view, Gdk.EventKey event)
    {
        /*string key = Gdk.keyval_name(event.keyval);
        if(window.visible)
        {
            switch(key)
            {
            case "Tab":
                Gtk.TreeSelection selection = tree_view.get_selection();
                var model = (Gtk.TreeModel)store;
                Gtk.TreeIter iter;
                if(selection.get_selected(out model, out iter))
                {
                    if(!store.iter_next(ref iter))
                    {
                        store.get_iter_first(out iter);
                        selection.select_iter(iter);
                    }
                    else
                    {
                    selection.select_iter(iter);
                    }
                }
                else
                {
                    store.get_iter_first(out iter);
                    selection.select_iter(iter);
                }
                return true;
            case "Return":
                return on_selection_changed(tree_view, new Gtk.TreePath(), new Gtk.TreeViewColumn());
            }
        }*/
        if(timeout > 0) Source.remove(timeout);
        timeout = Timeout.add(100, () => { update_completion(); return false; });
        return false;
    }
    
    uint timeout = 0;
    
    string last_to_find;

    void update_completion()
    {
        /*lock(to_find)
        {
            if(window == null) create_window();

            /* Compute the string we want compute *
            to_find = "";
            Gtk.TextIter iter;
            Gtk.TextBuffer buffer = this.main_window.get_current_document ().source_view.buffer;
            buffer.get_iter_at_offset(out iter, buffer.cursor_position);
            bool stopper_point = false;
            iter.backward_find_char( (c) =>
            {
                bool valid = c in stoppers;
                if(!valid)
                    to_find += c.to_string();
                return valid;
            }, null);

            to_find = to_find.reverse();
            last_to_find = to_find;

            /* Get the coords of the cursor and move the window *
            int x;
            int y;
            Gdk.Rectangle rect = Gdk.Rectangle();
            this.main_window.get_current_document ().source_view.get_iter_location(iter, out rect);
            int buffer_x = rect.x;
            int buffer_y = rect.y;
            int real_x;
            int real_y;
            this.main_window.get_current_document ().source_view.buffer_to_window_coords(Gtk.TextWindowType.WIDGET, buffer_x, buffer_y, out real_x, out real_y);
            (this.main_window.get_current_document ().source_view as Gtk.Widget).get_window().get_origin(out x, out y);
            window.move(x + real_x - 2, y + real_y + 20);
            
            if(to_find == "")
            {
                window.hide();
                return;
            }

            store.clear();
            
            int count = 0;
            foreach(var word in parser.get_for_word(to_find))
            {
                Gtk.TreeIter iter_;
                store.append(out iter_, null);
                store.set(iter_, 0, word, 1, word, -1);
                debug (word);
                count ++;
            }
            if(count == 0)
            {
                window.hide();
                return;
            }
            update_window_size(count);
            
            window.show_all();
            this.main_window.get_current_document ().source_view.grab_focus();
        }*/
    }
}

[ModuleInit]
public void peas_register_types (GLib.TypeModule module) {
    var objmodule = module as Peas.ObjectModule;
    objmodule.register_extension_type (typeof (Peas.Activatable),
                                       typeof (Euclide.Plugins.Completion));
}
